package com.isu.gaswellwatch.vo.command.weps;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

/**
 * 威尔普斯控制指令
 *
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2025/1/10 11:00
 */
@Getter
@Setter
@SuperBuilder
public class MiWepsPlugControl extends Command implements Timing {
    @Serial
    private static final long serialVersionUID = -7540015879950965127L;

    //    private Integer gasWellNumber; // 气井编号
    private Integer depthOfLockDevice; // 卡定器深度 单位：米
    private Integer arrivingSensor; // 到达传感器 0/1：禁用/启用
    private Integer zigbee; // Zigbee 0/1：禁用/启用
    private Integer pulseWidth; // 脉冲宽度 驱动电磁阀使用
    private Integer plugRisingSpeed; // 上升速度 米/分钟
    private Integer plugTooFastSpeed; // 过快速度 米/分钟
    private Integer plugTooSlowSpeed; // 过缓速度 米/分钟
    //    private Integer gasWellName; // 气井名称
//    分段
    private Integer oilAndCasDataCollectionInterval; // 油、套压数据采集间隔 0：每5秒采样；1：每10秒采样；2：每15秒采样；3：每30秒采样；4：每1秒采样；5：每2秒采样；6：每3秒采样；7：每4秒采样
    private Integer storeDataIntervalOnOpenWell; // 数据存储间隔（开井期间） 0：每1分钟存储；1：每5分钟存储；2：每10分钟存储；3：每15分钟存储；4：每30秒存储；5：每15秒存储；6：每5秒存储
    private Integer storeDataIntervalOnCloseWell; // 数据存储间隔（关井期间） 0：每1分钟存储；1：每5分钟存储；2：每10分钟存储；3：每15分钟存储；4：每30秒存储；5：每15秒存储；6：每5秒存储
    //    分段
    //    private Integer communicationPowerThreshold; // 通信电量门限 Zigbee通讯使用
//    private Integer reserve1; // 保留 Zigbee通讯使用
//    private Integer startTimeOfDaytime; // 白天起始时间 Zigbee通讯使用
//    private Integer endTimeOfDaytime; // 白天结束时间 Zigbee通讯使用
    private Integer controlWorkTime; // 控制器工作状态 0：生产模式；1：常开模式；2：常关模式
    private Integer produceMode; // 生产制度设置 0：定时开关井；1：时间优化模式；2：压力微升模式；3：压力回升模式；4：压力跌落模式
    private String plungerRiseTime; // 上升时间 单位：秒
    private String dangerousRiseTime; // 危险上升时间 单位：秒
    private String tooFastRiseTime; // 过快上升时间 单位：秒
    private String tooSlowRiseTime; // 过缓上升时间 单位：秒
    private Integer tooFastCount; // 过快次数
    private String tooFastRaiseOpenWellTime; // 过快增加开井时间 单位：秒
    private String tooFastLowerCloseWellTime; // 过快减少关井时间 单位：秒
    private String tooSlowLowerOpenWellTime; // 过缓减少开井时间 单位：秒
    private String tooSlowRaiseCloseWellTime; // 过缓增加关井时间 单位：秒
    private String checkStabilityTime; // 检测稳定时间 单位：秒
    private String minCloseWellTime; // 最小关井时间 单位：秒
    private String maxCloseWellTime; // 最大关井时间 单位：秒
    private String wellCloseTimeNotReachedDuration; // 未到达关井时间 单位：秒
    private String minOpenWellTime; // 最小开井时间 单位：秒
    private String maxOpenWellTime; // 最大开井时间 单位：秒
    private String afterFlowTime; // 续流时间 单位：秒
    private String notReachedFlowTime; // 未到续流时间 单位：秒
    private String closeTime; // 关井时间 单位：秒
    private String openTime; // 开井时间 单位：秒
    private BigDecimal openPressure; // 开井压力 单位：MPa
    private BigDecimal closePressure; // 关井压力 单位：MPa
    private BigDecimal littleRisePressure; // 微升压力 单位：MPa
    private Integer gasCollectionMode; // 集气模式 0：低压集气模式；1：高压集气模式
    //    private Integer reserve2; // 保留
//    分段
    private BigDecimal protectionPressure; // 保护压力 单位：MPa

    public MiWepsPlugControl() {
        this.setCode("MI_WEPS.PLUG.CONTROL");
    }


    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {

        StringBuilder command = new StringBuilder(250);
        //               100
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     003F     0013    26
        command.append("0110003F001326");
//        command.append(StringUtils.leftPad(Integer.toHexString(this.runMode), 4, "0"));
//        command.append(StringUtils.leftPad(Integer.toHexString(this.wellStatus), 4, "0"));
//        command.append(StringUtils.leftPad(Integer.toHexString(this.gasCollectionMethod), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.highPressureProtection.multiply(ONE_HUNDRED).longValue()), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.lowPressureProtection.multiply(ONE_HUNDRED).longValue()), 4, "0"));
//        command.append(StringUtils.leftPad(Integer.toHexString(this.tooFastCount), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.dangerousRiseTime)), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.tooFastRiseTime)), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.openWellTime.multiply(ONE_SIXTY).longValue()), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.closeWellTime.multiply(ONE_SIXTY).longValue()), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.openWellCasPressure.multiply(ONE_HUNDRED).longValue()), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.littleRiseCasPressure.multiply(ONE_HUNDRED).longValue()), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.checkStabilityTime)), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.toMinutes(this.minCloseWellTime)), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.toMinutes(this.maxCloseWellTime)), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.toMinutes(this.minOpenWellTime)), 4, "0"));
//        command.append(StringUtils.leftPad(Long.toHexString(this.toMinutes(this.maxOpenWellTime)), 4, "0"));
//        command.append(StringUtils.leftPad(Integer.toHexString(this.wellDepth), 4, "0"));
//        command.append(StringUtils.leftPad(Integer.toHexString(this.produceMode), 4, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

}
